/*
** main.c
**
** Written by Laurentiu Piciu - 2017
**
*/
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include "lcd.h"

#define BTN_UP 		(1 << PB5)
#define BTN_LEFT 	(1 << PB4)
#define BTN_DOWN	(1 << PB6)
#define BTN_RIGHT	(1 << PB7)
#define BTN_UP_PRESSED		((PINB & BTN_UP) == 0)
#define BTN_LEFT_PRESSED	((PINB & BTN_LEFT) == 0)
#define BTN_DOWN_PRESSED	((PINB & BTN_DOWN) == 0)
#define BTN_RIGHT_PRESSED	((PINB & BTN_RIGHT) == 0)

typedef enum {
	RIGHT = 1,
	LEFT = 2,
	UP = 3,
	DOWN = 4
} direction;

#define EASY		3
#define MEDIUM1		4
#define MEDIUM2		5
#define HARD		6

#define LCD_HT 	6
#define LCD_WH  84

#define MAX_LEN 150
#define INITIAL_X	44
#define INITIAL_Y	20

direction curr_direction = RIGHT;
unsigned char game_mode = EASY;
unsigned char pixels[LCD_WH][LCD_HT] = {0};

int length = 0;
unsigned char head1_x = 0, tail2_x = 0, head1_y = 0, tail2_y = 0;
unsigned char body_x[MAX_LEN + 4] = {0};
unsigned char body_y[MAX_LEN + 4] = {0};

void start_new_game()
{
	while (1) {
		lcd_goto_xy_exact(15, 2);
		lcd_str("Press any");
		lcd_goto_xy_exact(15, 3);
		lcd_str("button to");
		lcd_goto_xy_exact(15, 4);
		lcd_str("start the");
		lcd_goto_xy_exact(30, 5);
		lcd_str("game!");
		_delay_ms(500);
		lcd_clear();
		if (BTN_UP_PRESSED || BTN_DOWN_PRESSED || BTN_LEFT_PRESSED || BTN_RIGHT_PRESSED)
			break;
	}
}

void draw_main_menu()
{
	unsigned char y_arrow = 3;
	while (1) {
		lcd_goto_xy_exact(15, 1);
		lcd_str("Difficulty");
		lcd_goto_xy_exact(15, 3);
		lcd_str("Easy");
		lcd_goto_xy_exact(15, 4);
		lcd_str("Medium 1");
		lcd_goto_xy_exact(15, 5);
		lcd_str("Medium 2");
		lcd_goto_xy_exact(15, 6);
		lcd_str("Hard");
		lcd_goto_xy_exact(5, y_arrow);
		lcd_chr(ARROW_RIGHT);
		_delay_ms(500);
		lcd_clear();
		if (BTN_DOWN_PRESSED) {
			if (y_arrow == 6)
				y_arrow = 3;
			else
				y_arrow++;
		}
		
		if (BTN_UP_PRESSED) {
			if (y_arrow == 3)
				y_arrow = 6;
			else
				y_arrow--;
		}
		
		if (BTN_RIGHT_PRESSED) {
			game_mode = y_arrow;
			break;
		}
	}
}

void draw_pixel(unsigned char x, unsigned char y, unsigned char val)
{
	lcd_goto_xy_exact(x, y);
	lcd_base_addr();
	lcd_send(0xFF, LCD_DATA);
}

void set_pixel(unsigned char x, unsigned char y, unsigned char value) {
	char poz_y_bloc = (y-1) / 8 + 1;
	char poz_y_bit = (y-1) % 8;
	if (value == 1)
		pixels[x-1][poz_y_bloc-1] |= 1 << poz_y_bit;
	else
		pixels[x-1][poz_y_bloc-1] &= ~(1 << poz_y_bit);
	lcd_goto_xy_exact(x, poz_y_bloc);
	lcd_base_addr();
	lcd_send(pixels[x-1][poz_y_bloc-1], LCD_DATA);
}

void draw_frame()
{
	int i;
	lcd_goto_xy_exact(1, 1);
	lcd_base_addr();
	for (i = 1; i <= LCD_WH; ++i)
		lcd_send(0x01, LCD_DATA);
	
	lcd_goto_xy_exact(1, LCD_HT);
	lcd_base_addr();
	for (i = 1; i <= LCD_WH; ++i)
		lcd_send(0x80, LCD_DATA);
	
	for (i = 1 ; i <= LCD_HT ; i++) {
		draw_pixel(1, i, 3);
		draw_pixel(LCD_WH, i, 4);
	}
	
	if (game_mode == EASY || game_mode == MEDIUM1)
		return;
	
	for (i = 1; i <= LCD_HT - 2; ++i) {
		draw_pixel(10, i, 5);
		draw_pixel(11, i, 5);
		draw_pixel(73, i, 5);
		draw_pixel(74, i, 5);
	}
	
	for (i = 3; i <= LCD_HT; ++i) {
		draw_pixel(30, i, 5);
		draw_pixel(31, i, 5);
		draw_pixel(53, i, 5);
		draw_pixel(54, i, 5);
	}
}

void move(direction d)
{
	int i;
	if (d == RIGHT)
		head1_x++;
	else if (d == LEFT)
		head1_x--;
	else if (d == UP)
		head1_y--;
	else if (d == DOWN)
		head1_y++;
	set_pixel(head1_x, head1_y, 1);
	set_pixel(tail2_x, tail2_y, 0);

	for (i = 1 ; i < length ; i++) {
		body_x[i-1] = body_x[i];
		body_y[i-1] = body_y[i];
	}
	body_x[length-1] = head1_x;
	body_y[length-1] = head1_y;
	tail2_x = body_x[0];
	tail2_y = body_y[0];
}

void draw_snake()
{
	int i = 0;
	for (i = 0; i < 4; ++i) {
		length++;
		body_x[length-1] = INITIAL_X - i;
		body_y[length-1] = INITIAL_Y;
		set_pixel(body_x[length-1], body_y[length-1], 1);
	}
	
	head1_x = body_x[length-1];
	head1_y = body_y[length-1];
	
	tail2_x = body_x[0];
	tail2_y = body_y[0];
	
	while (1) {
		if (BTN_UP_PRESSED && curr_direction != DOWN)
			curr_direction = UP;
		if (BTN_DOWN_PRESSED && curr_direction != UP)
			curr_direction = DOWN;
		if (BTN_LEFT_PRESSED && curr_direction != RIGHT)
			curr_direction = LEFT;
		if (BTN_RIGHT_PRESSED && curr_direction != LEFT)
			curr_direction = RIGHT;
			
		move(curr_direction);
		_delay_ms(200);
	}
}

/* main -- Program starts here */
int main(void)
{
	// Setup LCD	
	lcd_init();
	lcd_contrast(0x40);
	
	//setez butoanele
	DDRB &= ~(1 << PB7);
	DDRB &= ~(1 << PB6);
	DDRB &= ~(1 << PB5);
	DDRB &= ~(1 << PB4);

	/* Activez rezistenta de pull-up pentru butoane. */
	PORTB |= (1 << PB7);
	PORTB |= (1 << PB6);
	PORTB |= (1 << PB5);
	PORTB |= (1 << PB4);

	//start_new_game();
	//draw_main_menu();
	draw_frame();
	//draw_snake();
}
